import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class VendView extends SCView {

	private VendingMachine vendingMachine_;
	public VendingMachine vendingMachine() { return vendingMachine_; }
	public void vendingMachine( VendingMachine aVendingMachine ) { vendingMachine_ = aVendingMachine; }

    VendView() {
        super("VendingMachine");
        this.initialize();
    }

    public static void main( String args[] ) {
        VendView aVendView = new VendView();
        aVendView.show();
    }


    private void initialize() {

        this.vendingMachine(new VendingMachine());

        this.initMenu();
        this.initLabels();
        this.initButtons();
        this.initCustomButtons();
        this.initMiscelaneous();

        this.hookupObservers();

        this.setSize(631,520);
        this.setLocation(20,20);


    }
    private void initMenu(){
        MenuBar aMenuBar = new MenuBar();

        Menu aMenu = new Menu("File");
        MenuItem aMenuItem = new MenuItem("Exit");
        aMenuItem.addActionListener(this);
        aMenuItem.setActionCommand("Exit");
        aMenu.add(aMenuItem);
        aMenuBar.add(aMenu);

        this.setMenuBar(aMenuBar);

    }
    private void initLabels(){
        Font bigFont = new Font("Helvetica",Font.BOLD,20);
        Font mediumFont = new Font("Helvetica",Font.BOLD,14);
        Font smallFont = new Font("Helvetica",Font.BOLD,10);

        Label aLabel = new Label("Acme Vending Machine");
        aLabel.setFont( bigFont );
        aLabel.setSize(240,50);
        aLabel.setLocation(50,70);
        this.addWidget("acmeVendingMachineLabel",aLabel);

        aLabel = new Label("Sold Out");
        aLabel.setFont( mediumFont );
        aLabel.setSize(80,30);
        aLabel.setLocation(105,155);
        this.addWidget("soldOutLabel",aLabel);

        aLabel = new Label("Exact Change Required");
        aLabel.setFont( mediumFont );
        aLabel.setSize(180,30);
        aLabel.setLocation(340,75);
        this.addWidget("exactChangeRequiredLabel",aLabel);

        aLabel = new Label("Total Deposited");
        aLabel.setFont( mediumFont );
        aLabel.setSize(110,25);
        aLabel.setLocation(390,320);
        this.addWidget("totalDeposited",aLabel);

        aLabel = new ObserverLabel("$0.00");
        aLabel.setFont( bigFont );
        aLabel.setAlignment(aLabel.CENTER);
        aLabel.setSize(100,25);
        aLabel.setLocation(510,275);
        aLabel.setBackground(new Color(200,200,200));
        this.addWidget("totalDepositedLabel",aLabel);

        aLabel = new ObserverLabel("$0.00");
        aLabel.setFont( bigFont );
        aLabel.setAlignment(aLabel.CENTER);
        aLabel.setSize(100,25);
        aLabel.setLocation(400,415);
        aLabel.setBackground(new Color(200,200,200));
        this.addWidget("changeLabel",aLabel);

    }
    private void initButtons(){
        this.addWidget("ddButton",new DispenseDrinkButton(this));
        this.addWidget("crbButton",new CRButton(this));
    }

    private void initCustomButtons(){
        this.addWidget("quarterButton", new QuarterButton(this));
        this.addWidget("dimeButton", new DimeButton(this));
        this.addWidget("nickelButton", new NickelButton(this));

    }
    private void initMiscelaneous(){
        this.addWidget("drinkDispensedCanvas", new DrinkDispensedCanvas(this));
        this.addWidget("coinSlotCanvas", new CoinSlotCanvas(this));
        this.addWidget("soldOutCanvas", new SoldOutCanvas(this));
        this.addWidget("exactChangeCanvas", new ExactChangeCanvas(this));

    }
    public void hookupObservers() {

        this.vendingMachine().moneySlot().addObserver(
            (CoinSlotCanvas) this.getWidget("coinSlotCanvas"));

        this.vendingMachine().dispenser().dispenserGate().addObserver(
            (DrinkDispensedCanvas) this.getWidget("drinkDispensedCanvas"));

        this.vendingMachine().dispenser().soldOutSensor().soldOutLite().addObserver(
            (SoldOutCanvas) this.getWidget("soldOutCanvas"));

        this.vendingMachine().moneySlot().coinManager().addObserver(
            (ObserverLabel) this.getWidget("totalDepositedLabel"));

       this.vendingMachine().coinReturnButton().returnGate().addObserver(
            (ObserverLabel) this.getWidget("changeLabel"));

        this.vendingMachine().moneySlot().coinManager().changeDispenser().addObserver(
            (ObserverLabel) this.getWidget("changeLabel"));

        this.vendingMachine().dispenser().coinManager().changeDispenser().changeSensor().exactChangeLite().addObserver(
            (ExactChangeCanvas) this.getWidget("exactChangeCanvas"));


    }

    public void dispenseDrinkButtonPressed(){
        System.out.println("dispenseDrinkButton in VendView");
        this.vendingMachine().dispenser().dispenseButtonPressed();
    }
    public void coinReturnButtonPressed(){
        System.out.println("coinReturnButton in VendView");
        this.vendingMachine().coinReturnButton().press();

    }
    public void quarterButtonPressed(){
        System.out.println("quarterButton in VendView");
        this.vendingMachine().moneySlot().coinReceived(
            new Coin(25) );
    }
    public void dimeButtonPressed(){
        System.out.println("dimeButton in VendView");
        this.vendingMachine().moneySlot().coinReceived(
            new Coin(10) );
    }
    public void nickelButtonPressed(){
        System.out.println("nickelButton in VendView");
        this.vendingMachine().moneySlot().coinReceived(
            new Coin(5) );
    }
    public void update(Observable obs, Object o) {
        String theString = (String) o;
        System.out.println(">> " + theString);
    }
    public void actionPerformed(ActionEvent ae){
        String actionCommand = ae.getActionCommand();
        if( actionCommand == "Exit" ){
            this.exit();
        }
    }
    public void windowClosing(WindowEvent we){
        this.exit();
    }
    private void exit(){
        System.out.println("closing VendView");
        this.delay();
        System.exit(0);
    }
    public void delay() {
        try { Thread.sleep(1000); }
        catch (InterruptedException e) { System.out.println ("View.delay() - uh..oh");
        }
    }

}
